<?php
/**
* @package Joomla
* @subpackage Contact Form
* @copyright Copyright 2007 Brian Katzung, Kappa Computer Solutions, LLC.
* Portions copyright (C) 2005 Open Source Matters. All rights reserved.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
* Joomla! is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/

// no direct access
defined( '_VALID_MOS' ) or die( 'Restricted access' );

global $task, $mainframe, $gid;

// load the HTML drawing classes
require_once( $mainframe->getPath( 'front_html' ) );
require_once( $mainframe->getPath( 'front_html', 'com_content' ) );
require_once( dirname( __FILE__ ) . '/correctvalue.php' );
cf_language();

$access = new stdClass();
$access->canEdit = false;
$access->canEditOwn = false;
$access->canPublish = false;

// cache activation for content portion
$cache =& mosCache::getCache( 'com_content' );

// configuration parameters
$menu = $mainframe->get( 'menu' );
$mparms =& new mosParameters( $menu->params );
$mparms->def( 'content_id', -1 );

$row = cf_get_content( $mparms->get( 'content_id' ), $gid );
$params =& new mosParameters( $row->attribs? $row->attribs: '' );

$params->def( 'pageclass_sfx', $mparms->get( 'pageclass_sfx', '' ) );
$params->def( 'page_title', $mparms->get( 'page_title', 1 ) );
$params->def( 'page_header', $mparms->get( 'page_header', $menu->name ) );
$params->def( 'form_header', $mparms->get( 'form_header', '' ) );
$params->def( 'back_button',
  $mparms->get( 'back_button', $mainframe->getCfg( 'back_button' ) ) );
$params->set( 'content_id', $mparms->get( 'content_id' ) );
$params->set( 'information', $mparms->get( 'information', 'intro' ) );
$params->set( 'confirmation', $mparms->get( 'confirmation', 'full' ) );
$params->set( 'sendmail', $mparms->def( 'sendmail', '/usr/sbin/sendmail -t' ) );
$params->set( 'default_domain',
  $mparms->def( 'default_domain', 'localhost.localdomain' ) );
$params->set( 'sender', $mparms->def( 'sender', 'webmaster' ) );
$params->set( 'store_mesg', $mparms->def( 'store_mesg', 0 ) );
$params->set( 'embed_content', $mparms->def( 'embed_content', 0 ) );
$params->set( 'include_variables', $mparms->def( 'include_variables', '' ) );
$params->set( 'send_doc', $mparms->get( 'send_doc', '' ) );
$params->set( 'doc_ref', $mparms->get( 'doc_ref', '' ) );
$params->set( 'contact_list',
  $mparms->def( 'contact_list', "General Delivery {info}\nWebmaster {webmaster}" ) );
$params->set( 'field_list', $mparms->def( 'field_list',
  "*name company csz phone+ fax *email2 subject message senddoc" ) );
$params->set( 'image', 1 );

// run-time parameters
$params->def( 'status', '' );

// page title
if ($params->get( 'page_title' ))
	$mainframe->setPageTitle( $params->get( 'page_header' ) );

switch ($task)
{
case 'send':
	if (cf_send_message( $params ))
		cf_show_confirmation( $row, $params );
	else
		cf_show_form( $row, $params );
	break;

default:
	cf_show_form( $row, $params );
	break;
}

//----------------------------------------------------------------------
// Return the contact list as rows of (label, value, email@domain)

function cf_contact_list ( $params )
{
	$raw_rows = preg_split( "/(?:<br\s*\/?>|[\r\n])+/", $params->get( 'contact_list' ) );
	$rows = array();

	foreach ($raw_rows as $row)
	{
		if (preg_match( "/^(.*?)\s*\{(.*?)(?:@(.*))?\}/", $row, $matches ))
		{
			$label = $matches[1];
			$email = $matches[2];
			$domain = ($matches[3] != '')? $matches[3]:
			  $params->get( 'default_domain' );
			$value = md5($row);
			$rows[] = array( $label, $value, "$email@$domain" );
		}
	}

	return $rows;
}

//----------------------------------------------------------------------
// Return selected content row
// (Based on components/com_content/content.php:showItem)

function &cf_get_content ( $uid, $gid )
{
	global $database, $mainframe;

	$now 		= _CURRENT_SERVER_TIME;
	$nullDate 	= $database->getNullDate();

	$xwhere = " AND ( a.state = 1 OR a.state = -1 )"
	. "\n AND ( a.publish_up = " . $database->Quote( $nullDate ) . " OR a.publish_up <= " . $database->Quote( $now ) . " )"
	. "\n AND ( a.publish_down = " . $database->Quote( $nullDate ) . " OR a.publish_down >= " . $database->Quote( $now ) . " )"
	;

	// main query
	$query = "SELECT a.*, u.name AS author, u.usertype, cc.name AS category, s.name AS section, g.name AS groups,"
	. "\n s.published AS sec_pub, cc.published AS cat_pub, s.access AS sec_access, cc.access AS cat_access,"
	. "\n s.id AS sec_id, cc.id as cat_id"
	. "\n FROM #__content AS a"
	. "\n LEFT JOIN #__categories AS cc ON cc.id = a.catid"
	. "\n LEFT JOIN #__sections AS s ON s.id = cc.section AND s.scope = 'content'"
	. "\n LEFT JOIN #__users AS u ON u.id = a.created_by"
	. "\n LEFT JOIN #__groups AS g ON a.access = g.id"
	. "\n WHERE a.id = " . (int) $uid
	. $xwhere
	. "\n AND a.access <= " . (int) $gid
	;
	$database->setQuery( $query );
	$row = NULL;

	if ( $database->loadObject( $row ) ) {
		/*
		* check whether category is published
		*/
		if ( !$row->cat_pub && $row->catid ) {
			return NULL;
		}
		/*
		* check whether section is published
		*/
		if ( !$row->sec_pub && $row->sectionid ) {
			return NULL;
		}
		/*
		* check whether category access level allows access
		*/
		if ( ($row->cat_access > $gid) && $row->catid ) {
			return NULL;
		}
		/*
		* check whether section access level allows access
		*/
		if ( ($row->sec_access > $gid) && $row->sectionid ) {
			return NULL;
		}
	}

	/*
	 * Note: The contact form must be published to be visible, but for
	 * our purposes, the content itself can and should be unpublished.
	 */
	return $row;
}

//----------------------------------------------------------------------
// Include language file

function cf_language ()
{
	global $mosConfig_absolute_path, $mosConfig_lang;

	$lang = "$mosConfig_absolute_path/administrator/components/com_contactform/language";
	$p = "$lang/$mosConfig_lang.php";
	if ( file_exists( $p ) )
	{
		require_once( $p );
	}
	else
	{
		$p = "$lang/english.php";
		if ( file_exists( $p ) )
		{
			require_once( $p );
		}
	}
}

//----------------------------------------------------------------------
// Try to send the message. Redisplay the form if there are errors.

function cf_send_message ( &$params )
{
	global $mainframe;

	#
	# Raw parameters
	#
	$ref = correctvalue( mosGetParam( $_REQUEST, 'ref', $_SERVER['HTTP_REFERER'] ) );
	$rmt_ip = $_SERVER['REMOTE_ADDR'];
	$agent = correctvalue( $_SERVER['HTTP_USER_AGENT'] );
	$to = correctvalue( mosGetParam( $_REQUEST, 'to', '' ) );
	$title = correctvalue( mosGetParam( $_REQUEST, 'title' ) );
	$name = correctvalue( mosGetParam( $_REQUEST, 'name' ) );
	$jobtitle = correctvalue( mosGetParam( $_REQUEST, 'jobtitle' ) );
	$company = correctvalue( mosGetParam( $_REQUEST, 'company' ) );
	$addr1 = correctvalue( mosGetParam( $_REQUEST, 'addr1' ) );
	$addr2 = correctvalue( mosGetParam( $_REQUEST, 'addr2' ) );
	$city = correctvalue( mosGetParam( $_REQUEST, 'city' ) );
	$state = correctvalue( mosGetParam( $_REQUEST, 'state' ) );
	$zip = correctvalue( mosGetParam( $_REQUEST, 'zip' ) );
	$phone = correctvalue( mosGetParam( $_REQUEST, 'phone', '' ) );
	$fax = correctvalue( mosGetParam( $_REQUEST, 'fax', '' ) );
	$emaddr = correctvalue( $_REQUEST['emaddr'] );
	$emaddr2 = correctvalue( $_REQUEST['emaddr2'] );
	$time = correctvalue( mosGetParam( $_REQUEST, 'time', '' ) );
	$subject = correctvalue( mosGetParam( $_REQUEST, 'subject', '' ) );
	$send_doc = correctvalue( mosGetParam( $_REQUEST, 'send_doc', '' ) );
	$mesg = correctvalue( mosGetParam( $_REQUEST, 'message', '' ) );

	#
	# Sanitized parameters
	#
	$use_name = preg_replace( '/[^a-z0-9\.,\' &\/-]/i', '', $name );
	$use_phone = preg_replace( '/[^0-9]/', '', $phone );
	$use_fax = preg_replace( '/[^0-9]/', '', $fax );
	preg_match( '/([a-z0-9_\.\/\?+-]+@[a-z0-9\.-]+)/i', $emaddr, $use_emaddr );
	$use_emaddr = $use_emaddr[1];
	$use_subject = preg_replace( '/[^ -~]/', '', $subject);

	#
	# Select recipient
	#
	$contact_list = cf_contact_list( $params );
	if ( count( $contact_list ) == 1 )
	{
		$contact_info = $contact_list[0];
		$to = $contact_info[1];
	}
	else
	{
		$contact_info = null;
		foreach ($contact_list as $contact)
		{
			if ($to == $contact[1])
			{
				$contact_info = $contact;
				break;
			}
		}
	}

	#
	# Determine selected fields
	#
	$fields = array();
	$field_list = explode( ' ', $params->get( 'field_list' ) );
	foreach ( $field_list as $field )
		$fields[$field] = 1;

	#
	# Validate parameters
	#
	if (!$contact_info)
	{
		$params->set( 'status', '<p class="cf_error">' .
		  CF_UNKNOWN_RECIPIENT . '</p>' );
		return false;
	}

	#
	# Message may only be blank if free document is requested
	#
	if ($mesg == '' && !$send_doc)
	{
		$params->set( 'status', '<p class="cf_error">' .
		  CF_MESSAGE_REQD . '</p>' );
		return false;
	}

	#
	# A valid phone number must be at least ten digits.
	#
	if ($phone != '' && strlen( $use_phone ) < 10)
	{
		$params->set( 'status', '<p class="cf_error">' .
		  CF_PHONE_INVALID . '</p>' );
		return false;
	}

	if ($fax != '' && strlen( $use_fax ) < 10)
	{
		$params->set( 'status', '<p class="cf_error">' .
		  CF_FAX_INVALID . '</p>' );
		return false;
	}

	#
	# Email must contain "@dom.ain".
	#
	if ($emaddr != '' && !preg_match('/@.+\../', $use_emaddr))
	{
		$params->set( 'status', '<p class="cf_error">' .
		  CF_EMAIL_INVALID . "<br />use_emaddr = " . htmlspecialchars($use_emaddr) . '</p>' );
		return false;
	}

	#
	# Make sure second email (if present) matches first.
	#
	if (($fields['email2'] || $fields['*email2']) && $emaddr != $emaddr2)
	{
		$params->set( 'status', '<p class="cf_error">' .
		  CF_EMAIL_MISMATCH . '</p>' );
		return false;
	}

	#
	# Check for required fields.
	#
	foreach (array(
		array('name', $name, CF_NAME_REQD),
		array('email', $emaddr, CF_EMAIL_REQD),
		array('email2', $emaddr, CF_EMAIL_REQD),
		array('email', $use_emaddr, CF_EMAIL_INVALID),
		array('email2', $use_emaddr, CF_EMAIL_INVALID),
		array('address', $address, CF_ADDRESS_REQD),
		array('csz', $address, CF_ADDRESS_REQD),
		array('csz', $city, CF_INVALID_ADDRESS),
		array('phone', $phone, CF_PHONE_REQD),
		array('phone+', $phone, CF_PHONE_REQD),
		array('fax', $fax, CF_FAX_REQD),
		array('subject', $subject, CF_SUBJECT_REQD)
	) as $check)
	{
		if ($fields["*{$check[0]}"] && $check[1] == '')
		{
			$params->set( 'status', "<p class='cf_error'>{$check[2]}</p>" );
			return false;
		}
	}

	#
	# Include name in subject line.
	#
	if ($use_name != '')
		$use_subject = $use_subject? "$use_name: $use_subject": $use_name;

	$def_domain = $params->get( 'default_domain' );
	$sender = explode( '@', $params->get( 'sender' ) );
	$sender = join( '@', array($sender[0], $sender[1]? $sender[1]: $def_domain) );
	$recipient = $contact_info[2];
	$from = "\"$contact_info[0] Contact Form\" <$sender>";
	$reply_to = ($use_emaddr != '')? (($use_name != '')? "\"$use_name\" <$use_emaddr>": $use_emaddr): '';

	$db_sender = ($use_name != '' && $use_emaddr != '')?
	  "\"$use_name\" <$use_emaddr>": "$use_name$use_emaddr";
	$db_recipient = $contact_info[0];

	$doc_ref = $params->get( 'doc_ref' );
	$send_doc = $send_doc? ($doc_ref? "yes ($doc_ref)": 'yes'): 'no';

	$content = array();
	array_push($content, <<<MAIL
From: $from\r
To: $recipient\r
Subject: $use_subject\r
X-Sender-IP: $rmt_ip\r

MAIL
	);
	if ($reply_to != '')
		array_push($content, "Reply-To: $reply_to\r\n" );
	array_push($content, <<<MAIL
\r
====================
Rmt IP:   $rmt_ip
Referrer: $ref
Agent:    $agent


MAIL
	);
	if ($title != '')
		array_push($content, "Title:     $title\n" );
	array_push($content,         "Name:      $name\n" );
	if ($jobtitle != '')
		array_push($content, "Job title: $jobtitle\n" );
	if ($company != '')
		array_push($content, "Company:   $company\n" );
	if ($addr1 != '')
	{
		array_push($content, "Address:   $addr1\n" );
		if ($addr2 != '')
			array_push($content, "           $addr2\n" );
	}
	if ($city != '')
		array_push($content, "City:      $city\n" );
	if ($state != '')
		array_push($content, "State:     $state\n" );
	if ($zip != '')
		array_push($content, "ZIP:       $zip\n" );
	if ($phone != '')
		array_push($content, "Phone:     $phone\n" );
	if ($time != '')
		array_push($content, "Time:      $time\n" );
	if ($fax != '')
		array_push($content, "FAX:       $fax\n" );
	if ($emaddr != '')
		array_push($content, "Email:     $emaddr\n" );
	if ($subject != '')
		array_push($content, "Subject:   $subject\n" );
	if ($params->get( 'send_doc' ))
		array_push($content, "Send doc:  $send_doc\n");
	array_push($content, "Message:\n$mesg\n" );

	if ( $incvs = trim( $params->get( 'include_variables' ) ) )
	{
		$incvs = preg_split( '/[\s,]+/', $incvs, -1, PREG_SPLIT_NO_EMPTY );
		array_push($content, "\n== Form Values ==\n\n" );

		if (!($query = $mainframe->get( 'CGI' )))
		{
			require_once('CGI.php');
			$mainframe->set( 'CGI', $query = new CGI() );
		}

		$pnames = $query->param();
		sort( $pnames );
		foreach ( $pnames as $pname )
		{
			switch ( $pname )
			{
			case 'to': case 'name': case 'company':
			case 'title': case 'jobtitle':
			case 'addr1': case 'addr2':
			case 'city': case 'state': case 'zip':
			case 'emaddr': case 'emaddr2':
			case 'phone': case 'fax': case 'time':
			case 'subject': case 'message': case 'send_doc':
			case 'ref':
				break;

			default:
				foreach ( $incvs as $incv )
					if ( substr( $pname, 0, strlen( $incv ) ) == $incv )
					{
						array_push($content, "$pname: " . $query->param( $pname ) . "\n" );
						break;
					}
				break;
			}
		}
	}

	$content = join( '', $content );
	$result = true;

	if ( $params->get( 'store_mesg' )  &&
	  !cf_store_message( $db_sender, $db_recipient, $subject, $content ) )
		$result = false;

	if ( $sendmail = $params->get( 'sendmail' ) )
	{
		$sfd = popen($sendmail, 'w');
		if ($sfd === false)
			$result = false;
		else
		{
			fputs( $sfd, $content );
			fputs( $sfd, "\n== End of Message ==\n" );
			pclose( $sfd );
		}
	}

	if ($result == $false)
		$params->set( 'status', '<p class="cf_error">' .
		  CF_SEND_FAILED . '</p>' );

	return $result;
}

//----------------------------------------------------------------------
// Display the confirmation

function cf_show_confirmation ( &$row, &$params )
{
	global $cache, $access;

	switch ($params->get( 'confirmation' ))
	{
	case 'intro':
		$row->text = $row->introtext;
		break;

	case 'full':
		$row->text = $row->fulltext;
		break;

	default:
		$row->text = $row->introtext . $row->fulltext;
		break;
	}

	cf_show_content( $row, $params );
}

//----------------------------------------------------------------------
// Show content
// (Based on components/com_content/content.php:show)

function cf_show_content ( &$row, &$params )
{
	global $access, $cache, $mainframe;

	// GC Parameters
	$params->def( 'link_titles', 	$mainframe->getCfg( 'link_titles' ) );
	$params->def( 'author', 	!$mainframe->getCfg( 'hideAuthor' ) );
	$params->def( 'createdate', 	!$mainframe->getCfg( 'hideCreateDate' ) );
	$params->def( 'modifydate', 	!$mainframe->getCfg( 'hideModifyDate' ) );
	$params->set( 'print', 		0 );
	$params->set( 'pdf', 		0 );
	$params->set( 'email', 		0 );
	$params->def( 'rating', 	0 );
	$params->def( 'icons', 		$mainframe->getCfg( 'icons' ) );
	$params->def( 'readmore', 	0 );

	if ($back = $params->get( 'back_button' ))
		$params->set( 'back_button', 0 );
	$row->state = 1;
	$row->hits = 0;
	$cache->call('HTML_content::show', $row, $params, $access, 0);
	if ($back)
		$params->set( 'back_button', 1 );
}

//----------------------------------------------------------------------
// Display the contact form

function cf_show_form ( &$row, &$params )
{
	switch ($params->get( 'information' ))
	{
	case 'intro':
		$row->text = $row->introtext;
		break;

	case 'full':
		$row->text = $row->fulltext;
		break;

	default:
		$row->text = '';
		break;
	}

	HTML_contactform::display_start( $params );
	cf_show_content( $row, $params );
	HTML_contactform::display( $params, cf_contact_list( $params ) );
}

//----------------------------------------------------------------------
// Store message in database

function cf_store_message ( $sender, $recipient, $subject, $message )
{
	global $database;

	$row = new stdClass();
	$row->when = strftime('%Y-%m-%d %T');
	$row->sender = $sender;
	$row->recipient = $recipient;
	$row->subject = $subject;
	$row->message = $message;

	return $database->insertObject( '#__cfmessage', $row );
}

//----------------------------------------------------------------------
// Encode in quoted-printable format

function quoted_printable_encode ($s)
{
	return preg_replace( '/[^\x09\x20-\x3C\x3E-\x7E]/e',
	  'sprintf( "=%02X", ord ( "$0" ) ) ;', $s );
}

// END
